<?php
/**
* The editing form code for this question type.
*
* @copyright &copy; 2009 Eric Brisson
* @author ebrisson at winona.edu
* @license http://www.gnu.org/copyleft/gpl.html GNU Public License
* @package question-type-interval
*/

require_once($CFG->dirroot.'/question/type/edit_question_form.php');

/**
* Intervals editing form definition.
* 
*/

class question_edit_interval_form extends question_edit_form {
	
    /**
    * Add question-type specific form fields.
    *
    * @param MoodleQuickForm $mform the form being built.
    */
    function definition_inner(&$mform) {
		
		$mform->removeElement('image');
		
        $mform->addElement('select', 'direction', get_string('direction','qtype_interval'),
            array( "a"  => get_string('dirasc', 'qtype_interval'),
            "d"  => get_string('dirdesc', 'qtype_interval')
            ));
		
        $mform->setHelpButton('direction',
            array('direction',get_string('direction','qtype_interval'),'qtype_interval'));
		
        $mform->addElement('select', 'quality', get_string('quality','qtype_interval'),
            array( "M"  => get_string('major','qtype_interval'),
            "m"  => get_string('minor','qtype_interval'),
            "P"  => get_string('perfect','qtype_interval'),
            "A"  => get_string('augmented','qtype_interval'),
            "D"  => get_string('diminished','qtype_interval'),
            ));
		
        $mform->setHelpButton('quality',
            array('quality',get_string('quality','qtype_interval'),'qtype_interval'));
		
		$mform->addElement('select', 'size', get_string('size','qtype_interval'), 
            array( "2"  => "2",
            "3"  => "3",
            "4"  => "4",
            "5"  => "5",
            "6"  => "6",
            "7"  => "7",
            "8"  => "8",
            "9"  => "9",
            "10"  => "10",
            "11"  => "11",
            "12"  => "12",
            "13"  => "13",
            ));
		
        $mform->setHelpButton('size',array('size',get_string('size','qtype_interval'),
            'qtype_interval'));
		
        $mform->addElement('select', 'orignoteletter', 
            get_string('orignoteletter','qtype_interval'),
            array( "C"  => get_string('C','qtype_interval'),
            "D"  => get_string('D','qtype_interval'),
            "E"  => get_string('E','qtype_interval'),
            "F"  => get_string('F','qtype_interval'),
            "G"  => get_string('G','qtype_interval'),
            "A"  => get_string('A','qtype_interval'),
            "B"  => get_string('B','qtype_interval'),
            ));
		
        $mform->setHelpButton('orignoteletter',
            array('orignoteletter',get_string('orignoteletter','qtype_interval'),'qtype_interval'));
		
		
        $mform->addElement('select', 'orignoteaccidental', 
            get_string('orignoteaccidental','qtype_interval'),
            array( ""  => "&#9838",
            "#"  => "&#9839",
            "b"  => "&#9837",
            "x"  => "x",
            "bb"  => "bb",
            ));
		
        $mform->setHelpButton('orignoteaccidental',
            array('orignoteaccidental',get_string('orignoteaccidental','qtype_interval'),
            'qtype_interval'));
		
        $mform->addElement('select', 'orignoteregister', 
            get_string('orignoteregister','qtype_interval'),
            array( "3"  => "3",
            "4"  => "4",
            "5"  => "5",
            ));
		
        $mform->setHelpButton('orignoteregister',
            array('orignoteregister',get_string('orignoteregister','qtype_interval'),
            'qtype_interval'));
		
        $mform->addElement('select', 'clef', get_string('clef','qtype_interval'),
            array( "t"  => get_string('treble','qtype_interval'),
            "b"  => get_string('bass','qtype_interval'),
            ));
		
        $mform->setHelpButton('clef',array('clef',get_string('clef','qtype_interval'),
            'qtype_interval'));
		
            $creategrades = get_grade_options();
            $this->add_per_answer_fields($mform, get_string('answerno', 'qtype_interval', '{no}'),
            $creategrades->gradeoptions, 1, 1);
    }
	
	
    /**
    * Overidden function. See Comments from the Base Class.
    * 
    * In this implementation, a help button is added to the answer box.
    */
    function get_per_answer_fields(&$mform, $label, $gradeoptions, &$repeatedoptions, &$answersoption) {
        $repeated = array();
        $repeated[] =& $mform->createElement('header', 'answerhdr', $label);
        $repeated[] =& $mform->createElement('text', 'answer', get_string('answer', 'quiz'), array('size' => 4));
        $repeated[] =& $mform->createElement('select', 'fraction', get_string('grade'), $gradeoptions);
        $repeated[] =& $mform->createElement('htmleditor', 'feedback', get_string('feedback', 'quiz'),
            array('course' => $this->coursefilesid));
		
        $repeatedoptions['answer']['type'] = PARAM_RAW;
        $repeatedoptions['fraction']['default'] = 0;
        $repeatedoptions['answer']['helpbutton'] = array('answer', get_string('answer','qtype_interval'),
            'qtype_interval');
        $answersoption = 'answers';
        return $repeated;
    }
	
	
    /**
    * Overidden function. See Comments from the Base Class.
    * 
    * In this implementation, the call to repeat_elements includes a custom string for 
    * the "Blanks for more Choices" button.
    */
    function add_per_answer_fields(&$mform, $label, $gradeoptions, 
    $minoptions = QUESTION_NUMANS_START, $addoptions = QUESTION_NUMANS_ADD) {
        $answersoption = '';
        $repeatedoptions = array();
        $repeated = $this->get_per_answer_fields($mform, $label, $gradeoptions, $repeatedoptions, $answersoption);
		
        if (isset($this->question->options)){
            $countanswers = count($this->question->options->$answersoption);
            } else {
            $countanswers = 0;
        }
        if ($this->question->formoptions->repeatelements){
            $repeatsatstart = max($minoptions, $countanswers + $addoptions);
            } else {
            $repeatsatstart = $countanswers;
        }
		
        $this->repeat_elements($repeated, $repeatsatstart, $repeatedoptions, 'noanswers', 'addanswers', 
        $addoptions, get_string('addAnswer','qtype_interval'), true);
    }
	
	
    /**
    * Sets the existing values into the form for the question specific data.
    * 
    * Originally comes from short answer question type. It sets the answers 
    * before calling the parent function.
    * 
    * Overidden function.
    *
    * @param $question the question object from the database being used to fill the form
    */
    function set_data($question) {
        if (isset($question->options)){
            $answers = $question->options->answers;
            if (count($answers)) {
                $key = 0;
                foreach ($answers as $answer){
                    $default_values['answer['.$key.']'] = $answer->answer;
                    $default_values['fraction['.$key.']'] = $answer->fraction;
                    $default_values['feedback['.$key.']'] = $answer->feedback;
                    $key++;
                }
            }
            $question = (object)((array)$question + $default_values);
        }
        parent::set_data($question);
    }
	
    /**
    * Overidden function. See Comments from the Base Class.
    */
    function validation($data, $files) {
        $errors = parent::validation($data, $files);
		
        if(($data['quality']=="M" || $data['quality']=="m") &&
        ($data['size']==1 || 
        $data['size']==4 || 
        $data['size']==5 ||
        $data['size']==8 ||
        $data['size']==11 ||
        $data['size']==12)
        ) {
            $errors['quality']=get_string('qualitymismatch','qtype_interval');
        }
		
        if(($data['quality']=="P") &&
        ($data['size']==2 || 
        $data['size']==3 || 
        $data['size']==6 ||
        $data['size']==7 ||
        $data['size']==9 ||
        $data['size']==10 ||
        $data['size']==13)
        ) {
            $errors['quality']=get_string('qualitymismatch','qtype_interval');
        }
		
        $answers = $data['answer'];
        $answercount = 0;
        $maxgrade = false;
        foreach ($answers as $key => $answer) {
            $trimmedanswer = trim($answer);
            if ($trimmedanswer !== ''){
                $answercount++;
                if ($data['fraction'][$key] == 1) {
                    $maxgrade = true;
                }
                } else if ($data['fraction'][$key] != 0 || !html_is_blank($data['feedback'][$key])) {
                $errors["answer[$key]"] = get_string('answermustbegiven', 'qtype_shortanswer');
                $answercount++;
            }
        }
        if ($answercount==0){
            $errors['answer[0]'] = get_string('notenoughanswers', 'quiz', 1);
        }
        if ($maxgrade == false) {
            $errors['fraction[0]'] = get_string('fractionsnomax', 'question');
        }
        return $errors;
		
		
        if ($errors) {
            return $errors;
            } else {
            return true;
        }
    }
	
	
    function qtype() {
	    return 'interval';
    }
}
?>